#property link          "https://www.earnforex.com/metatrader-indicators/draw-grid/"
#property version       "1.01"
#property strict
#property copyright     "EarnForex.com - 2019-2021"
#property description   "This Indicator will allow you to draw a grid" 
#property description   "specifying a leg and sub leg."
#property description   " "
#property description   "WARNING : You use this software at your own risk."
#property description   "The creator of these plugins cannot be held responsible for damage or loss."
#property description   " "
#property description   "Find More on EarnForex.com"
#property icon          "\\Files\\EF-Icon-64x64px.ico"

#property indicator_chart_window

#include <MQLTA ErrorHandling.mqh>
#include <MQLTA Utils.mqh>

enum ENUM_THICKNESS{
   One=1,      //1
   Two=2,      //2
   Three=3,    //3
   Four=4,     //4
   Five=5,     //5
};


input string Comment_1="====================";           //Draw Grid Settings
input string IndicatorName="DG";                         //Indicator Name (used to draw objects)

input string Comment_2="====================";           //Grid Settings
extern double StartPrice=0;                              //Start Price (0=Current)
extern double LowRange=0;                                //Minimum Value (0=Low of the whole chart)
extern double HighRange=0;                               //Maximum Value (0=High of the whole chart)

extern int MainLeg=1000;                                 //Main Grid Leg (Points)
input ENUM_LINE_STYLE MainStyle=STYLE_SOLID;             //Main Grid Style
input color MainColor=clrRoyalBlue;                      //Main Grid Color
input ENUM_THICKNESS MainThick=Two;                      //Main Grid Thickness

extern bool ShowSubGrid=true;                            //Show Secondary Grid
extern int SubLeg=250;                                   //Secondary Grid Leg (Points)
input ENUM_LINE_STYLE SubStyle=STYLE_DASH;               //Secondary Grid Style
input color SubColor=clrRoyalBlue;                       //Secondary Grid Color
input ENUM_THICKNESS SubThick=One;                       //Secondary Grid Thickness

input string Comment_4="====================";           //Panel Position
extern int Xoff=20;                                      //Horizontal spacing for the control panel
extern int Yoff=20;                                      //Vertical spacing for the control panel

string Font="Consolas";
int FontSize=8;
int eDigits=0;

int OnInit(){
   IndicatorSetString(INDICATOR_SHORTNAME,IndicatorName);

   eDigits=(int)MarketInfo(Symbol(),MODE_DIGITS);
   CleanChart();
   ChartSetInteger(0,CHART_EVENT_MOUSE_MOVE,1);
   CreateMiniPanel();
   return(INIT_SUCCEEDED);
   
}


int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]){

   return(rates_total);
}


void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam){
   
   
   if(id==CHARTEVENT_OBJECT_CLICK){
      if(sparam==PanelExp){
         DrawGrid();
      }
      if(sparam==PanelDel){
         CleanGrid();
      }
      if(sparam==PanelOptions){
         ShowSettings();
      }
      if(sparam==SettingsClose){
         CloseSettings();
      }
      if(sparam==SettingsSave){
         SaveSettingsChanges();
      }
      if(sparam==SettingsShowSubE){
         ChangeShowSub();
      }
   }
   
   if(id==CHARTEVENT_MOUSE_MOVE){
      if(StringToInteger(sparam)==1){
         if(lparam>(Xoff+2) && lparam<(Xoff+2+PanelLabX) &&
            dparam>(Yoff+2) && dparam<(Yoff+2+PanelLabY)){
            Xoff=(int)lparam-2-PanelLabX/2;
            Yoff=(int)dparam-2-PanelLabY/2;
            CloseSettings();
            UpdatePanel();
         }
      }
   }
   
   
}


void OnDeinit(const int reason){
   CleanGrid();
   CleanChart();
}


void CleanChart(){
   int Window=0;
   for(int i=ObjectsTotal(ChartID(),Window,-1)-1;i>=0;i--){
      if(StringFind(ObjectName(i),IndicatorName,0)>=0 && !(StringFind(ObjectName(i),IndicatorName+"-HLINE-",0)>=0)){
         ObjectDelete(ObjectName(i));
      }
   }
}


void CleanGrid(){
   int Window=0;
   for(int i=ObjectsTotal(ChartID(),Window,-1)-1;i>=0;i--){
      if(StringFind(ObjectName(i),IndicatorName+"-HLINE-",0)>=0){
         ObjectDelete(ObjectName(i));
      }
   }
}


void CleanMiniPanel(){
   int Window=0;
   for(int i=ObjectsTotal(ChartID(),Window,-1)-1;i>=0;i--){
      if(StringFind(ObjectName(i),IndicatorName+"-P-",0)>=0){
         ObjectDelete(ObjectName(i));
      }
   }
}


void DrawGrid(){
   CleanGrid();
   CloseSettings();
   double GridPrice=0;
   double Points=MarketInfo(Symbol(),MODE_POINT);
   double MainStep=MainLeg*Points;
   double SubStep=SubLeg*Points;
   if(HighRange==0) HighRange=(High[iHighest(Symbol(),0,MODE_HIGH,WHOLE_ARRAY,0)])+MainStep;
   if(LowRange==0) LowRange=(Low[iLowest(Symbol(),0,MODE_LOW,WHOLE_ARRAY,0)])-MainStep;
   if(LowRange<0) LowRange=0;
   if(StartPrice==0) StartPrice=Close[0];
   if(StartPrice>HighRange || StartPrice<LowRange){
      Print("The Start Price must be between the Minimum and the Maximum Values");
      Print("Start Price: ",StartPrice," - Minimum: ",LowRange," - Maximum: ",HighRange);
      return;
   }
   GridPrice=StartPrice;
   while(GridPrice<=HighRange){
      string LineName=IndicatorName+"-HLINE-M-"+IntegerToString((int)MathRound(GridPrice/Points));
      if(ObjectFind(0,LineName)>=0){
         GridPrice+=MainStep;
         continue;
      } 
      ObjectCreate(0,LineName,OBJ_HLINE,0,0,GridPrice);
      ObjectSet(LineName,OBJPROP_COLOR,MainColor);
      ObjectSet(LineName,OBJPROP_STYLE,MainStyle);
      ObjectSet(LineName,OBJPROP_WIDTH,MainThick);
      ObjectSet(LineName,OBJPROP_SELECTABLE,false);
      GridPrice+=MainStep;
   }
   GridPrice=StartPrice;
   while(GridPrice>=LowRange){
      string LineName=IndicatorName+"-HLINE-M-"+IntegerToString((int)MathRound(GridPrice/Points));
      if(ObjectFind(0,LineName)>=0){
         GridPrice-=MainStep;
         continue;
      }
      ObjectCreate(0,LineName,OBJ_HLINE,0,0,GridPrice);
      ObjectSet(LineName,OBJPROP_COLOR,MainColor);
      ObjectSet(LineName,OBJPROP_STYLE,MainStyle);
      ObjectSet(LineName,OBJPROP_WIDTH,MainThick);
      ObjectSet(LineName,OBJPROP_SELECTABLE,false);
      GridPrice-=MainStep;
   }
   if(ShowSubGrid){
      GridPrice=StartPrice;
      while(GridPrice<=HighRange){
         string LineName=IndicatorName+"-HLINE-S-"+IntegerToString((int)MathRound(GridPrice/Points));
         if(ObjectFind(0,LineName)>=0){
            GridPrice+=SubStep;
            continue;
         }
         ObjectCreate(0,LineName,OBJ_HLINE,0,0,GridPrice);
         ObjectSet(LineName,OBJPROP_COLOR,SubColor);
         ObjectSet(LineName,OBJPROP_STYLE,SubStyle);
         ObjectSet(LineName,OBJPROP_WIDTH,SubThick);
         ObjectSet(LineName,OBJPROP_SELECTABLE,false);
         GridPrice+=SubStep;
      }
      GridPrice=StartPrice;
      while(GridPrice>=LowRange){
         string LineName=IndicatorName+"-HLINE-S-"+IntegerToString((int)MathRound(GridPrice/Points));
         if(ObjectFind(0,LineName)>=0){
            GridPrice-=SubStep;
            continue;
         }
         ObjectCreate(0,LineName,OBJ_HLINE,0,0,GridPrice);
         ObjectSet(LineName,OBJPROP_COLOR,SubColor);
         ObjectSet(LineName,OBJPROP_STYLE,SubStyle);
         ObjectSet(LineName,OBJPROP_WIDTH,SubThick);
         ObjectSet(LineName,OBJPROP_SELECTABLE,false);
         GridPrice+=SubStep;
      }
   }
   CreateMiniPanel();
   return;
}


string PanelBase=IndicatorName+"-P-BAS";
string PanelLabel=IndicatorName+"-P-LAB";
string PanelExp=IndicatorName+"-P-EXP";
string PanelDel=IndicatorName+"-P-DEL";
string PanelOptions=IndicatorName+"-P-OPT";
string PanelURL=IndicatorName+"-P-URL";

int PanelMovX=26;
int PanelMovY=26;
int PanelLabX=101;
int PanelLabY=PanelMovY;
int PanelRecX=(PanelMovX+2)*3+PanelLabX+1;

void CreateMiniPanel(){
   CleanMiniPanel();
   ObjectCreate(0,PanelBase,OBJ_RECTANGLE_LABEL,0,0,0);
   ObjectSet(PanelBase,OBJPROP_XDISTANCE,Xoff);
   ObjectSet(PanelBase,OBJPROP_YDISTANCE,Yoff);
   ObjectSetInteger(0,PanelBase,OBJPROP_XSIZE,PanelRecX);
   ObjectSetInteger(0,PanelBase,OBJPROP_YSIZE,(PanelMovY+2)*1+2);
   ObjectSetInteger(0,PanelBase,OBJPROP_BGCOLOR,White);
   ObjectSetInteger(0,PanelBase,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,PanelBase,OBJPROP_STATE,false);
   ObjectSetInteger(0,PanelBase,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,PanelBase,OBJPROP_FONTSIZE,8);
   ObjectSet(PanelBase,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,PanelBase,OBJPROP_COLOR,clrBlack);
      
   ObjectCreate(0,PanelLabel,OBJ_EDIT,0,0,0);
   ObjectSet(PanelLabel,OBJPROP_XDISTANCE,Xoff+2);
   ObjectSet(PanelLabel,OBJPROP_YDISTANCE,Yoff+2);
   ObjectSetInteger(0,PanelLabel,OBJPROP_XSIZE,PanelLabX);
   ObjectSetInteger(0,PanelLabel,OBJPROP_YSIZE,PanelLabY);
   ObjectSetInteger(0,PanelLabel,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,PanelLabel,OBJPROP_STATE,false);
   ObjectSetInteger(0,PanelLabel,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,PanelLabel,OBJPROP_READONLY,true);
   ObjectSetInteger(0,PanelLabel,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,PanelLabel,OBJPROP_TOOLTIP,"Drag to Move");
   ObjectSetString(0,PanelLabel,OBJPROP_TEXT,"DRAW GRID");
   ObjectSetString(0,PanelLabel,OBJPROP_FONT,"Consolas");
   ObjectSetInteger(0,PanelLabel,OBJPROP_FONTSIZE,12);
   ObjectSet(PanelLabel,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,PanelLabel,OBJPROP_COLOR,clrNavy);
   ObjectSetInteger(0,PanelLabel,OBJPROP_BGCOLOR,clrKhaki);
   ObjectSetInteger(0,PanelLabel,OBJPROP_BORDER_COLOR,clrBlack);

   ObjectCreate(0,PanelExp,OBJ_EDIT,0,0,0);
   ObjectSet(PanelExp,OBJPROP_XDISTANCE,Xoff+PanelLabX+3);
   ObjectSet(PanelExp,OBJPROP_YDISTANCE,Yoff+2);
   ObjectSetInteger(0,PanelExp,OBJPROP_XSIZE,PanelMovX);
   ObjectSetInteger(0,PanelExp,OBJPROP_YSIZE,PanelMovX);
   ObjectSetInteger(0,PanelExp,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,PanelExp,OBJPROP_STATE,false);
   ObjectSetInteger(0,PanelExp,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,PanelExp,OBJPROP_READONLY,true);
   ObjectSetInteger(0,PanelExp,OBJPROP_FONTSIZE,12);
   ObjectSetString(0,PanelExp,OBJPROP_TOOLTIP,"Click to Draw the Grid");
   ObjectSetInteger(0,PanelExp,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,PanelExp,OBJPROP_FONT,"Wingdings");
   ObjectSetString(0,PanelExp,OBJPROP_TEXT,"!");
   ObjectSet(PanelExp,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,PanelExp,OBJPROP_COLOR,clrNavy);
   ObjectSetInteger(0,PanelExp,OBJPROP_BGCOLOR,clrKhaki);
   ObjectSetInteger(0,PanelExp,OBJPROP_BORDER_COLOR,clrBlack);

   ObjectCreate(0,PanelDel,OBJ_EDIT,0,0,0);
   ObjectSet(PanelDel,OBJPROP_XDISTANCE,Xoff+PanelLabX+(PanelMovX+2)*1+2);
   ObjectSet(PanelDel,OBJPROP_YDISTANCE,Yoff+2);
   ObjectSetInteger(0,PanelDel,OBJPROP_XSIZE,PanelMovX);
   ObjectSetInteger(0,PanelDel,OBJPROP_YSIZE,PanelMovX);
   ObjectSetInteger(0,PanelDel,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,PanelDel,OBJPROP_STATE,false);
   ObjectSetInteger(0,PanelDel,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,PanelDel,OBJPROP_READONLY,true);
   ObjectSetInteger(0,PanelDel,OBJPROP_FONTSIZE,12);
   ObjectSetString(0,PanelDel,OBJPROP_TOOLTIP,"Click to Delete Current Grid");
   ObjectSetInteger(0,PanelDel,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,PanelDel,OBJPROP_FONT,"Wingdings");
   ObjectSetString(0,PanelDel,OBJPROP_TEXT,"I");
   ObjectSet(PanelDel,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,PanelDel,OBJPROP_COLOR,clrNavy);
   ObjectSetInteger(0,PanelDel,OBJPROP_BGCOLOR,clrKhaki);
   ObjectSetInteger(0,PanelDel,OBJPROP_BORDER_COLOR,clrBlack);

   ObjectCreate(0,PanelOptions,OBJ_EDIT,0,0,0);
   ObjectSet(PanelOptions,OBJPROP_XDISTANCE,Xoff+(PanelMovX+2)*2+PanelLabX+1);
   ObjectSet(PanelOptions,OBJPROP_YDISTANCE,Yoff+2);
   ObjectSetInteger(0,PanelOptions,OBJPROP_XSIZE,PanelMovX);
   ObjectSetInteger(0,PanelOptions,OBJPROP_YSIZE,PanelMovX);
   ObjectSetInteger(0,PanelOptions,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,PanelOptions,OBJPROP_STATE,false);
   ObjectSetInteger(0,PanelOptions,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,PanelOptions,OBJPROP_READONLY,true);
   ObjectSetInteger(0,PanelOptions,OBJPROP_FONTSIZE,12);
   ObjectSetInteger(0,PanelOptions,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,PanelOptions,OBJPROP_FONT,"Wingdings");
   ObjectSetString(0,PanelOptions,OBJPROP_TOOLTIP,"Click to Open the Options Panel");
   ObjectSetString(0,PanelOptions,OBJPROP_TEXT,":");
   ObjectSet(PanelOptions,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,PanelOptions,OBJPROP_COLOR,clrNavy);
   ObjectSetInteger(0,PanelOptions,OBJPROP_BGCOLOR,clrKhaki);
   ObjectSetInteger(0,PanelOptions,OBJPROP_BORDER_COLOR,clrBlack);
   
}


void UpdatePanel(){
   ObjectSet(PanelBase,OBJPROP_XDISTANCE,Xoff);
   ObjectSet(PanelBase,OBJPROP_YDISTANCE,Yoff);
   ObjectSet(PanelLabel,OBJPROP_XDISTANCE,Xoff+2);
   ObjectSet(PanelLabel,OBJPROP_YDISTANCE,Yoff+2);
   ObjectSet(PanelExp,OBJPROP_XDISTANCE,Xoff+PanelLabX+3);
   ObjectSet(PanelExp,OBJPROP_YDISTANCE,Yoff+2);
   ObjectSet(PanelURL,OBJPROP_XDISTANCE,Xoff+2);
   ObjectSet(PanelDel,OBJPROP_XDISTANCE,Xoff+PanelLabX+(PanelMovX+2)*1+2);
   ObjectSet(PanelDel,OBJPROP_YDISTANCE,Yoff+2);
   ObjectSet(PanelOptions,OBJPROP_XDISTANCE,Xoff+(PanelMovX+2)*2+PanelLabX+1);
   ObjectSet(PanelOptions,OBJPROP_YDISTANCE,Yoff+2);
   ObjectSet(PanelURL,OBJPROP_YDISTANCE,Yoff+PanelLabY+2+1);
}




string SettingsBase=IndicatorName+"-S-Base";
string SettingsSave=IndicatorName+"-S-Save";
string SettingsClose=IndicatorName+"-S-Close";
string SettingsStartPrice=IndicatorName+"-S-Screenshot";
string SettingsStartPriceE=IndicatorName+"-S-ScreenshotE";
string SettingsLowRange=IndicatorName+"-S-LotSize";
string SettingsLowRangeE=IndicatorName+"-S-LotSizeE";
string SettingsHighRange=IndicatorName+"-S-LotStep";
string SettingsHighRangeE=IndicatorName+"-S-LotStepE";
string SettingsMainLeg=IndicatorName+"-S-RiskPerc";
string SettingsMainLegE=IndicatorName+"-S-RiskPercE";
string SettingsShowSub=IndicatorName+"-S-RiskBase";
string SettingsShowSubE=IndicatorName+"-S-RiskBaseE";
string SettingsSubLeg=IndicatorName+"-S-OrderPerPage";
string SettingsSubLegE=IndicatorName+"-S-OrderPerPageE";

int SetGLabelX=100;
int SetGLabelEX=80;
int SetGLabelY=20;
int SetButtonX=90;
int SetButtonY=SetGLabelY;

void ShowSettings(){
   int SetXoff=Xoff;
   int SetYoff=Yoff+PanelMovY*1+6;
   int SetX=SetButtonX*2+6;
   int SetY=(SetButtonY+2)*7+2;
   int j=1;
   string TextStartPrice="";
   string TextLowRange="";
   string TextHighRange="";
   string TextMainLeg=IntegerToString(MainLeg);
   string TextShowSub="";
   string TextSubLeg=IntegerToString(SubLeg);
   if(ShowSubGrid) TextShowSub="ON";
   else TextShowSub="OFF";
   if(HighRange==0) TextHighRange=DoubleToStr(High[iHighest(Symbol(),0,MODE_HIGH,WHOLE_ARRAY,0)],eDigits);
   else TextHighRange=DoubleToStr(HighRange,eDigits);
   if(LowRange==0) TextLowRange=DoubleToStr(Low[iLowest(Symbol(),0,MODE_LOW,WHOLE_ARRAY,0)],eDigits);
   else TextLowRange=DoubleToStr(LowRange,eDigits);
   if(LowRange<0) LowRange=0;
   if(StartPrice==0) TextStartPrice=DoubleToStr(Close[0],eDigits);
   else TextStartPrice=DoubleToStr(StartPrice,eDigits);
   
   ObjectCreate(0,SettingsBase,OBJ_RECTANGLE_LABEL,0,0,0);
   ObjectSet(SettingsBase,OBJPROP_XDISTANCE,SetXoff);
   ObjectSet(SettingsBase,OBJPROP_YDISTANCE,SetYoff);
   ObjectSetInteger(0,SettingsBase,OBJPROP_XSIZE,SetX);
   ObjectSetInteger(0,SettingsBase,OBJPROP_YSIZE,SetY);
   ObjectSetInteger(0,SettingsBase,OBJPROP_BGCOLOR,White);
   ObjectSetInteger(0,SettingsBase,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsBase,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsBase,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsBase,OBJPROP_FONTSIZE,8);
   ObjectSet(SettingsBase,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsBase,OBJPROP_COLOR,clrBlack);
      
   ObjectCreate(0,SettingsSave,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsSave,OBJPROP_XDISTANCE,SetXoff+2);
   ObjectSet(SettingsSave,OBJPROP_YDISTANCE,SetYoff+2);
   ObjectSetInteger(0,SettingsSave,OBJPROP_XSIZE,SetButtonX);
   ObjectSetInteger(0,SettingsSave,OBJPROP_YSIZE,SetButtonY);
   ObjectSetInteger(0,SettingsSave,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsSave,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsSave,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsSave,OBJPROP_READONLY,true);
   ObjectSetInteger(0,SettingsSave,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsSave,OBJPROP_TOOLTIP,"Save Changes");
   ObjectSetInteger(0,SettingsSave,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,SettingsSave,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsSave,OBJPROP_TEXT,"Save");
   ObjectSet(SettingsSave,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsSave,OBJPROP_COLOR,clrBlack);
   ObjectSetInteger(0,SettingsSave,OBJPROP_BGCOLOR,clrPaleGreen);
   ObjectSetInteger(0,SettingsSave,OBJPROP_BORDER_COLOR,clrBlack);
   
   ObjectCreate(0,SettingsClose,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsClose,OBJPROP_XDISTANCE,SetXoff+(SetButtonX+2)*1+2);
   ObjectSet(SettingsClose,OBJPROP_YDISTANCE,SetYoff+2);
   ObjectSetInteger(0,SettingsClose,OBJPROP_XSIZE,SetButtonX);
   ObjectSetInteger(0,SettingsClose,OBJPROP_YSIZE,SetButtonY);
   ObjectSetInteger(0,SettingsClose,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsClose,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsClose,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsClose,OBJPROP_READONLY,true);
   ObjectSetInteger(0,SettingsClose,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsClose,OBJPROP_TOOLTIP,"Close Settings Panel");
   ObjectSetInteger(0,SettingsClose,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,SettingsClose,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsClose,OBJPROP_TEXT,"X");
   ObjectSet(SettingsClose,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsClose,OBJPROP_COLOR,clrBlack);
   ObjectSetInteger(0,SettingsClose,OBJPROP_BGCOLOR,clrCrimson);
   ObjectSetInteger(0,SettingsClose,OBJPROP_BORDER_COLOR,clrBlack);
   
   ObjectCreate(0,SettingsStartPrice,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsStartPrice,OBJPROP_XDISTANCE,SetXoff+2);
   ObjectSet(SettingsStartPrice,OBJPROP_YDISTANCE,SetYoff+2+(SetButtonY+2)*j);
   ObjectSetInteger(0,SettingsStartPrice,OBJPROP_XSIZE,SetGLabelX);
   ObjectSetInteger(0,SettingsStartPrice,OBJPROP_YSIZE,SetGLabelY);
   ObjectSetInteger(0,SettingsStartPrice,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsStartPrice,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsStartPrice,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsStartPrice,OBJPROP_READONLY,true);
   ObjectSetInteger(0,SettingsStartPrice,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsStartPrice,OBJPROP_TOOLTIP,"Starting Price for the Grid");
   ObjectSetInteger(0,SettingsStartPrice,OBJPROP_ALIGN,ALIGN_LEFT);
   ObjectSetString(0,SettingsStartPrice,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsStartPrice,OBJPROP_TEXT,"Start Price");
   ObjectSet(SettingsStartPrice,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsStartPrice,OBJPROP_COLOR,clrBlack);

   ObjectCreate(0,SettingsStartPriceE,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsStartPriceE,OBJPROP_XDISTANCE,SetXoff+2+SetGLabelX+2);
   ObjectSet(SettingsStartPriceE,OBJPROP_YDISTANCE,SetYoff+2+(SetButtonY+2)*j);
   ObjectSetInteger(0,SettingsStartPriceE,OBJPROP_XSIZE,SetGLabelEX);
   ObjectSetInteger(0,SettingsStartPriceE,OBJPROP_YSIZE,SetButtonY);
   ObjectSetInteger(0,SettingsStartPriceE,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsStartPriceE,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsStartPriceE,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsStartPriceE,OBJPROP_READONLY,false);
   ObjectSetInteger(0,SettingsStartPriceE,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsStartPriceE,OBJPROP_TOOLTIP,"Start Price for the Grid - Click to change");
   ObjectSetInteger(0,SettingsStartPriceE,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,SettingsStartPriceE,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsStartPriceE,OBJPROP_TEXT,TextStartPrice);
   ObjectSet(SettingsStartPriceE,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsStartPriceE,OBJPROP_COLOR,clrBlack);
   j++;
   
   ObjectCreate(0,SettingsLowRange,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsLowRange,OBJPROP_XDISTANCE,SetXoff+2);
   ObjectSet(SettingsLowRange,OBJPROP_YDISTANCE,SetYoff+2+(SetButtonY+2)*j);
   ObjectSetInteger(0,SettingsLowRange,OBJPROP_XSIZE,SetGLabelX);
   ObjectSetInteger(0,SettingsLowRange,OBJPROP_YSIZE,SetGLabelY);
   ObjectSetInteger(0,SettingsLowRange,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsLowRange,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsLowRange,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsLowRange,OBJPROP_READONLY,true);
   ObjectSetInteger(0,SettingsLowRange,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsLowRange,OBJPROP_TOOLTIP,"Grid Lowest Value");
   ObjectSetInteger(0,SettingsLowRange,OBJPROP_ALIGN,ALIGN_LEFT);
   ObjectSetString(0,SettingsLowRange,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsLowRange,OBJPROP_TEXT,"Low Range");
   ObjectSet(SettingsLowRange,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsLowRange,OBJPROP_COLOR,clrBlack);

   ObjectCreate(0,SettingsLowRangeE,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsLowRangeE,OBJPROP_XDISTANCE,SetXoff+2+SetGLabelX+2);
   ObjectSet(SettingsLowRangeE,OBJPROP_YDISTANCE,SetYoff+2+(SetButtonY+2)*j);
   ObjectSetInteger(0,SettingsLowRangeE,OBJPROP_XSIZE,SetGLabelEX);
   ObjectSetInteger(0,SettingsLowRangeE,OBJPROP_YSIZE,SetButtonY);
   ObjectSetInteger(0,SettingsLowRangeE,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsLowRangeE,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsLowRangeE,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsLowRangeE,OBJPROP_READONLY,false);
   ObjectSetInteger(0,SettingsLowRangeE,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsLowRangeE,OBJPROP_TOOLTIP,"Grid Lowest Value - Click to change");
   ObjectSetInteger(0,SettingsLowRangeE,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,SettingsLowRangeE,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsLowRangeE,OBJPROP_TEXT,TextLowRange);
   ObjectSet(SettingsLowRangeE,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsLowRangeE,OBJPROP_COLOR,clrBlack);
   j++;
   
   ObjectCreate(0,SettingsHighRange,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsHighRange,OBJPROP_XDISTANCE,SetXoff+2);
   ObjectSet(SettingsHighRange,OBJPROP_YDISTANCE,SetYoff+2+(SetButtonY+2)*j);
   ObjectSetInteger(0,SettingsHighRange,OBJPROP_XSIZE,SetGLabelX);
   ObjectSetInteger(0,SettingsHighRange,OBJPROP_YSIZE,SetGLabelY);
   ObjectSetInteger(0,SettingsHighRange,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsHighRange,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsHighRange,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsHighRange,OBJPROP_READONLY,true);
   ObjectSetInteger(0,SettingsHighRange,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsHighRange,OBJPROP_TOOLTIP,"Grid Highest Value");
   ObjectSetInteger(0,SettingsHighRange,OBJPROP_ALIGN,ALIGN_LEFT);
   ObjectSetString(0,SettingsHighRange,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsHighRange,OBJPROP_TEXT,"High Range");
   ObjectSet(SettingsHighRange,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsHighRange,OBJPROP_COLOR,clrBlack);

   ObjectCreate(0,SettingsHighRangeE,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsHighRangeE,OBJPROP_XDISTANCE,SetXoff+2+SetGLabelX+2);
   ObjectSet(SettingsHighRangeE,OBJPROP_YDISTANCE,SetYoff+2+(SetButtonY+2)*j);
   ObjectSetInteger(0,SettingsHighRangeE,OBJPROP_XSIZE,SetGLabelEX);
   ObjectSetInteger(0,SettingsHighRangeE,OBJPROP_YSIZE,SetButtonY);
   ObjectSetInteger(0,SettingsHighRangeE,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsHighRangeE,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsHighRangeE,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsHighRangeE,OBJPROP_READONLY,false);
   ObjectSetInteger(0,SettingsHighRangeE,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsHighRangeE,OBJPROP_TOOLTIP,"Grid Highest Value - Click to change");
   ObjectSetInteger(0,SettingsHighRangeE,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,SettingsHighRangeE,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsHighRangeE,OBJPROP_TEXT,TextHighRange);
   ObjectSet(SettingsHighRangeE,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsHighRangeE,OBJPROP_COLOR,clrBlack);
   j++;
   
   ObjectCreate(0,SettingsMainLeg,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsMainLeg,OBJPROP_XDISTANCE,SetXoff+2);
   ObjectSet(SettingsMainLeg,OBJPROP_YDISTANCE,SetYoff+2+(SetButtonY+2)*j);
   ObjectSetInteger(0,SettingsMainLeg,OBJPROP_XSIZE,SetGLabelX);
   ObjectSetInteger(0,SettingsMainLeg,OBJPROP_YSIZE,SetGLabelY);
   ObjectSetInteger(0,SettingsMainLeg,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsMainLeg,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsMainLeg,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsMainLeg,OBJPROP_READONLY,true);
   ObjectSetInteger(0,SettingsMainLeg,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsMainLeg,OBJPROP_TOOLTIP,"Main Leg Step in Points");
   ObjectSetInteger(0,SettingsMainLeg,OBJPROP_ALIGN,ALIGN_LEFT);
   ObjectSetString(0,SettingsMainLeg,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsMainLeg,OBJPROP_TEXT,"Main Leg");
   ObjectSet(SettingsMainLeg,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsMainLeg,OBJPROP_COLOR,clrBlack);

   ObjectCreate(0,SettingsMainLegE,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsMainLegE,OBJPROP_XDISTANCE,SetXoff+2+SetGLabelX+2);
   ObjectSet(SettingsMainLegE,OBJPROP_YDISTANCE,SetYoff+2+(SetButtonY+2)*j);
   ObjectSetInteger(0,SettingsMainLegE,OBJPROP_XSIZE,SetGLabelEX);
   ObjectSetInteger(0,SettingsMainLegE,OBJPROP_YSIZE,SetButtonY);
   ObjectSetInteger(0,SettingsMainLegE,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsMainLegE,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsMainLegE,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsMainLegE,OBJPROP_READONLY,false);
   ObjectSetInteger(0,SettingsMainLegE,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsMainLegE,OBJPROP_TOOLTIP,"Main Leg Step in Points - Click to change");
   ObjectSetInteger(0,SettingsMainLegE,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,SettingsMainLegE,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsMainLegE,OBJPROP_TEXT,TextMainLeg);
   ObjectSet(SettingsMainLegE,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsMainLegE,OBJPROP_COLOR,clrBlack);
   j++;
   
   ObjectCreate(0,SettingsShowSub,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsShowSub,OBJPROP_XDISTANCE,SetXoff+2);
   ObjectSet(SettingsShowSub,OBJPROP_YDISTANCE,SetYoff+2+(SetButtonY+2)*j);
   ObjectSetInteger(0,SettingsShowSub,OBJPROP_XSIZE,SetGLabelX);
   ObjectSetInteger(0,SettingsShowSub,OBJPROP_YSIZE,SetGLabelY);
   ObjectSetInteger(0,SettingsShowSub,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsShowSub,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsShowSub,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsShowSub,OBJPROP_READONLY,true);
   ObjectSetInteger(0,SettingsShowSub,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsShowSub,OBJPROP_TOOLTIP,"Show Secondary Grid");
   ObjectSetInteger(0,SettingsShowSub,OBJPROP_ALIGN,ALIGN_LEFT);
   ObjectSetString(0,SettingsShowSub,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsShowSub,OBJPROP_TEXT,"Show Sub Leg");
   ObjectSet(SettingsShowSub,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsShowSub,OBJPROP_COLOR,clrBlack);

   ObjectCreate(0,SettingsShowSubE,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsShowSubE,OBJPROP_XDISTANCE,SetXoff+2+SetGLabelX+2);
   ObjectSet(SettingsShowSubE,OBJPROP_YDISTANCE,SetYoff+2+(SetButtonY+2)*j);
   ObjectSetInteger(0,SettingsShowSubE,OBJPROP_XSIZE,SetGLabelEX);
   ObjectSetInteger(0,SettingsShowSubE,OBJPROP_YSIZE,SetButtonY);
   ObjectSetInteger(0,SettingsShowSubE,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsShowSubE,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsShowSubE,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsShowSubE,OBJPROP_READONLY,true);
   ObjectSetInteger(0,SettingsShowSubE,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsShowSubE,OBJPROP_TOOLTIP,"Show Secondary Grid - Click to change");
   ObjectSetInteger(0,SettingsShowSubE,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,SettingsShowSubE,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsShowSubE,OBJPROP_TEXT,TextShowSub);
   ObjectSet(SettingsShowSubE,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsShowSubE,OBJPROP_COLOR,clrBlack);
   j++;
   
   ObjectCreate(0,SettingsSubLeg,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsSubLeg,OBJPROP_XDISTANCE,SetXoff+2);
   ObjectSet(SettingsSubLeg,OBJPROP_YDISTANCE,SetYoff+2+(SetButtonY+2)*j);
   ObjectSetInteger(0,SettingsSubLeg,OBJPROP_XSIZE,SetGLabelX);
   ObjectSetInteger(0,SettingsSubLeg,OBJPROP_YSIZE,SetGLabelY);
   ObjectSetInteger(0,SettingsSubLeg,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsSubLeg,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsSubLeg,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsSubLeg,OBJPROP_READONLY,true);
   ObjectSetInteger(0,SettingsSubLeg,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsSubLeg,OBJPROP_TOOLTIP,"Secondary Leg Step in Points");
   ObjectSetInteger(0,SettingsSubLeg,OBJPROP_ALIGN,ALIGN_LEFT);
   ObjectSetString(0,SettingsSubLeg,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsSubLeg,OBJPROP_TEXT,"Sub Leg");
   ObjectSet(SettingsSubLeg,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsSubLeg,OBJPROP_COLOR,clrBlack);

   ObjectCreate(0,SettingsSubLegE,OBJ_EDIT,0,0,0);
   ObjectSet(SettingsSubLegE,OBJPROP_XDISTANCE,SetXoff+2+SetGLabelX+2);
   ObjectSet(SettingsSubLegE,OBJPROP_YDISTANCE,SetYoff+2+(SetButtonY+2)*j);
   ObjectSetInteger(0,SettingsSubLegE,OBJPROP_XSIZE,SetGLabelEX);
   ObjectSetInteger(0,SettingsSubLegE,OBJPROP_YSIZE,SetButtonY);
   ObjectSetInteger(0,SettingsSubLegE,OBJPROP_BORDER_TYPE,BORDER_FLAT);
   ObjectSetInteger(0,SettingsSubLegE,OBJPROP_STATE,false);
   ObjectSetInteger(0,SettingsSubLegE,OBJPROP_HIDDEN,true);
   ObjectSetInteger(0,SettingsSubLegE,OBJPROP_READONLY,false);
   ObjectSetInteger(0,SettingsSubLegE,OBJPROP_FONTSIZE,FontSize);
   ObjectSetString(0,SettingsSubLegE,OBJPROP_TOOLTIP,"Secondary Leg Step in Points - Click to change");
   ObjectSetInteger(0,SettingsSubLegE,OBJPROP_ALIGN,ALIGN_CENTER);
   ObjectSetString(0,SettingsSubLegE,OBJPROP_FONT,Font);
   ObjectSetString(0,SettingsSubLegE,OBJPROP_TEXT,TextSubLeg);
   ObjectSet(SettingsSubLegE,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,SettingsSubLegE,OBJPROP_COLOR,clrBlack);
   j++;
   
}


void ChangeShowSub(){
   string Tmp=ObjectGetString(0,SettingsShowSubE,OBJPROP_TEXT);
   if(Tmp=="ON"){
      ObjectSetString(0,SettingsShowSubE,OBJPROP_TEXT,"OFF");
      return;
   }
   if(Tmp=="OFF"){
      ObjectSetString(0,SettingsShowSubE,OBJPROP_TEXT,"ON");
      return;
   }
}


void SaveSettingsChanges(){
   double SettingsStartPriceTmp=StringToDouble(ObjectGetString(0,SettingsStartPriceE,OBJPROP_TEXT));
   double SettingsLowRangeTmp=StringToDouble(ObjectGetString(0,SettingsLowRangeE,OBJPROP_TEXT));
   double SettingsHighRangeTmp=StringToDouble(ObjectGetString(0,SettingsHighRangeE,OBJPROP_TEXT));
   string SettingsShowSubTmp=ObjectGetString(0,SettingsShowSubE,OBJPROP_TEXT);
   int SettingsMainLegTmp=(int)StringToInteger(ObjectGetString(0,SettingsMainLegE,OBJPROP_TEXT));
   int SettingsSubLegTmp=(int)StringToInteger(ObjectGetString(0,SettingsSubLegE,OBJPROP_TEXT));
   if(SettingsStartPriceTmp<SettingsLowRangeTmp || SettingsStartPriceTmp>SettingsHighRangeTmp){
      Print("The Start Price must be between the Minimum and the Maximum Values");
      Print("Start Price: ",SettingsStartPriceTmp," - Minimum: ",SettingsLowRangeTmp," - Maximum: ",SettingsHighRangeTmp);
      ShowSettings();
      return;
   }
   StartPrice=SettingsStartPriceTmp;
   LowRange=SettingsLowRangeTmp;
   HighRange=SettingsHighRangeTmp;
   MainLeg=SettingsMainLegTmp;
   SubLeg=SettingsSubLegTmp;
   if(SettingsShowSubTmp=="ON") ShowSubGrid=true;
   else ShowSubGrid=false;   
   ShowSettings();
}


void CloseSettings(){
   int Window=0;
   for(int i=ObjectsTotal(ChartID(),Window,-1)-1;i>=0;i--){
      if((StringFind(ObjectName(i),IndicatorName+"-S-",0)>=0)){
         ObjectDelete(ObjectName(i));
      }
   }
}